import express, { Request, Response } from "express";
import { container } from "tsyringe";
import { Counter, Gauge, collectDefaultMetrics, register } from "prom-client";
import { LocationService } from "../services/locationService";
import { WifiLocationService } from "../services/wifiLocationService";
import { TtnGatewayReceptionService } from "../services/ttnGatewayReceptionService";
import { MetricsService } from "../services/metricsService";

const router = express.Router();
const locationService = container.resolve(LocationService);
const wifiLocationService = container.resolve(WifiLocationService);
const ttnGatewayReceptionService = container.resolve(TtnGatewayReceptionService);
const metricsService = container.resolve(MetricsService);

const requestCounter = metricsService.getRequestCounter();
const locationsTotal = metricsService.getLocationsTotal();
const gnssLocationsTotal = metricsService.getGnssLocationsTotal()
const wifiLocationTotal = metricsService.getWifiLocationTotal();
const wifiLocationNotResolvable = metricsService.getWifiLocationResolvable();
const wifiLocationRequestLimitExceeded = metricsService.getLWifiLocationRequestLimitExceeded();
const ttnGatewayReceptions = metricsService.getTnnGatewayReceptions();

// Define the metrics endpoint
router.get("/", async (req: Request, res: Response) => {
  try {
    const [
      allLocations,
      gnssLocations,
      allWifiLocations,
      wifiLocationsNotResolvable,
      wifiLocationsRequestLimitExceeded,
      allTtnGatewayReceptions
    ] = await Promise.all([
      locationService.getAllLocations(),
      locationService.getAllGnssLocations(),
      wifiLocationService.getAllWifiLocations(),
      wifiLocationService.getAllWifiLocationsByNotResolvable(),
      wifiLocationService.getAllWifiLocationsByRequestLimitExceeded(),
      ttnGatewayReceptionService.getAllGatewayReceptions()
    ]);

    locationsTotal.set(allLocations.length);
    gnssLocationsTotal.set(gnssLocations.length);
    wifiLocationTotal.set(allWifiLocations.length);
    wifiLocationNotResolvable.set(wifiLocationsNotResolvable.length);
    wifiLocationRequestLimitExceeded.set(wifiLocationsRequestLimitExceeded.length);
    ttnGatewayReceptions.set(allTtnGatewayReceptions.length);

    // Increment the counter with labels
    requestCounter.inc({ method: req.method, route: req.route.path, status: 200 });

    // Expose metrics in Prometheus format
    res.set("Content-Type", register.contentType);
    res.send(await register.metrics());
  } catch (error) {
    // Increment the counter for errors
    requestCounter.inc({ method: req.method, route: req.route.path, status: 500 });
    console.error("Error running metrics endpoint:", error);
    res.status(500).json({ error: "Error running metrics endpoint" });
  }
});

export default router;
